
// HTTPUtils.java
// Andrew Davison, ad@coe.psu.ac.th, July 2023

/* HTTP functions for use with Java's HttpClient
   Google GSON, and Desktop browser support.

  Usage:
    > compile HTTPUtils.java
*/

import java.awt.Desktop;
import java.io.*;
import java.util.*;
import java.net.*;
import java.net.http.*;
import java.nio.file.*;

import com.google.gson.*;


public class HTTPUtils
{

  public static String encode(String s)
  {
    try {
      return URLEncoder.encode(s, "UTF-8");
    }
    catch (Exception e) {
      System.out.println("Encoding error: " + e); 
      return s;
    }
  }  // end of encode()



  public static String getResponse(String request)
  // return a response string for the given request 
  {
    HttpStatus httpStatus = new HttpStatus();

    HttpClient client = HttpClient.newHttpClient();
    HttpRequest httpRequest = HttpRequest.newBuilder()
           .uri(URI.create(request))
           .GET().build();
    try {
      HttpResponse<String> response = client.send(httpRequest,
                HttpResponse.BodyHandlers.ofString());

      int status = response.statusCode();
      if (status != 200) {
        System.out.println("HTTP request failed due to a \"" +
                   httpStatus.getReason(status) + 
                   "\" error (status code: " + status + ")");
        return null;
      }
      else
        return response.body();
    }
    catch(Exception e) {
      System.out.println("HTTP request error: " + e);
      return null;
    }
  }  // end of getResponse()


  public static void prettyPrint(String json)
  {
    Gson gson = new GsonBuilder().setPrettyPrinting().create();
    JsonParser jp = new JsonParser();
    JsonElement je = jp.parse(json);
    System.out.println(gson.toJson(je));
  }


  public static ArrayList<String> getStringList(Object obj) 
  /* the input object can be string or a list of strings;
     always return a list */
  {
    ArrayList<String> ls = new ArrayList<>();
    if (obj == null)
      return ls;

    if (obj instanceof String) {
      ls.add((String)obj);
      return ls;
    }

    if (obj instanceof ArrayList<?>) {
      for (Object val : (ArrayList<?>) obj)
        ls.add((String) val);
      return ls;
    }

    System.out.println("Unknown type: " + obj);
    ls.add("??");
    return ls;
  }  // end of getStringList()



  public static void download(String url, String fnm)
  // download the URL and save it in a local file 
  {
    HttpStatus httpStatus = new HttpStatus();

    // HttpClient client = HttpClient.newHttpClient();
    HttpClient client = HttpClient.newBuilder()
        .followRedirects(HttpClient.Redirect.NORMAL)  // handle redirection
        .build();

    HttpRequest httpRequest = HttpRequest.newBuilder()
           .uri(URI.create(url))
           .GET().build();
    try {
      HttpResponse<InputStream> response = client.send(httpRequest,
                HttpResponse.BodyHandlers.ofInputStream());

      int status = response.statusCode();
      if (status != 200)  // not ok
        System.out.println("Request failed due to \"" +
                         httpStatus.getReason(status) + 
                         "\" (" + status + ")");
      else {
        System.out.println("Saving " + url + " to " + fnm);
        try {
          Files.copy(response.body(), Paths.get(fnm));
        }
        catch(IOException e)
        {  System.out.println("Unable to save: " + e);  }
      }
    }
    catch(Exception e) {
      System.out.println("HTTP request error: " + e);
    }
  }  // end of download()




  public static void showInBrowser(String url)
  // display the supplied URL in a browser
  /* JRE sometimes crashes due to a problematic frame:
     C  [ANSI64.dll+0x2887] in
     OpenJDK 64-Bit Server VM (build 19-panama+1-13, mixed mode, sharing)
  */
  {
    try {
      if (!Desktop.isDesktopSupported()) {
        System.out.println("Desktop access not supported");
        return;
      }
      Desktop desktop = Desktop.getDesktop();
      if (desktop == null)
        System.out.println("Could not access the desktop");
      else {
        if (desktop.isSupported(Desktop.Action.BROWSE)) {
          System.out.println("Displaying " + url + " in the browser");
          desktop.browse(new URI(url));
        }
        else
          System.out.println("Could not access the browser");
      }
    }
    catch(Exception e) {
      System.out.println("Desktop error: " + e);
    }
  }  // end of showInBrowser()



  public static String getParent(String url)
  // get the parent directory string for this URL
  { 
    try {
      URI uri = new URI(url);
      URI parent = uri.getPath().endsWith("/") ? uri.resolve("..") : 
                                                 uri.resolve(".");
      System.out.println("Parent directory: " + parent);
      return parent.toString();
    }
    catch (URISyntaxException e) {
      System.out.println(e);
      return url;
    }
  }  // end of getParent()


}  // end of HTTPUtils class
