
// HttpStatus.java
// Mike Bowler (mbowler@GargoyleSoftware.com) 
// and Jeff Dever (jsdever@apache.org)
// 29 Nov. 2006
// https://github.com/wso2/wso2-commons-httpclient/blob/master/commons-httpclient/src/main/java/org/apache/commons/httpclient/HttpStatus.java

/* Give reasons for all the status codes defined in 
   RFC1945 (HTTP/1.0, RFC2616 (HTTP/1.1), and
   RFC2518 (WebDAV).
*/


public class HttpStatus
{
  // Reasons for status codes 0xx - 5xx
  // array sizes == largest status code+1 in each 100 range
  private static final String[][] REASONS = new String[][] {
    new String[0],
    new String[3],   
    new String[8],
    new String[8],
    new String[25],
    new String[8]
  };



  public static String getReason(int statusCode)
  // return the reason associated with the status code
  {
    if (statusCode < 0)
      throw new IllegalArgumentException("status code may not be negative");

    int rowIdx = statusCode/100;
      // each row stores codes for a hundred range: 0xx - 5xx
    int codeIdx = statusCode - rowIdx*100;  // the xx value
    if ((rowIdx < 1) || (rowIdx > (REASONS.length - 1)) ||
        (codeIdx < 0) || (codeIdx > (REASONS[rowIdx].length - 1)) ) {
      System.out.println("No reason set for status: " + statusCode);
      return null;
    }
    return REASONS[rowIdx][codeIdx];
  } /* getReason() */


  private static void addStatus(int statusCode, String reason)
  // Store a reason for the given status code
  {
    int rowIdx = statusCode/100;
      // each row stores codes for a hundred range: 0xx - 5xx
    REASONS[rowIdx][statusCode - rowIdx*100] = reason;    
        // store the reason in the cell for the xx value
  }


  // --- 1xx Informational ---
  public static final int SC_CONTINUE = 100;
  public static final int SC_SWITCHING_PROTOCOLS = 101;
  public static final int SC_PROCESSING = 102;

  // --- 2xx Success ---
  public static final int SC_OK = 200;
  public static final int SC_CREATED = 201;
  public static final int SC_ACCEPTED = 202;
  public static final int SC_NON_AUTHORITATIVE_INFORMATION = 203;
  public static final int SC_NO_CONTENT = 204;
  public static final int SC_RESET_CONTENT = 205;
  public static final int SC_PARTIAL_CONTENT = 206;
  public static final int SC_MULTI_STATUS = 207;

  // --- 3xx Redirection ---
  public static final int SC_MULTIPLE_CHOICES = 300;
  public static final int SC_MOVED_PERMANENTLY = 301;
  public static final int SC_MOVED_TEMPORARILY = 302;
  public static final int SC_SEE_OTHER = 303;
  public static final int SC_NOT_MODIFIED = 304;
  public static final int SC_USE_PROXY = 305;
  public static final int SC_TEMPORARY_REDIRECT = 307;

  // --- 4xx Client Error ---
  public static final int SC_BAD_REQUEST = 400;
  public static final int SC_UNAUTHORIZED = 401;
  public static final int SC_PAYMENT_REQUIRED = 402;
  public static final int SC_FORBIDDEN = 403;
  public static final int SC_NOT_FOUND = 404;
  public static final int SC_METHOD_NOT_ALLOWED = 405;
  public static final int SC_NOT_ACCEPTABLE = 406;
  public static final int SC_PROXY_AUTHENTICATION_REQUIRED = 407;
  public static final int SC_REQUEST_TIMEOUT = 408;
  public static final int SC_CONFLICT = 409;
  public static final int SC_GONE = 410;
  public static final int SC_LENGTH_REQUIRED = 411;
  public static final int SC_PRECONDITION_FAILED = 412;
  public static final int SC_REQUEST_TOO_LONG = 413;
  public static final int SC_REQUEST_URI_TOO_LONG = 414;
  public static final int SC_UNSUPPORTED_MEDIA_TYPE = 415;
  public static final int SC_REQUESTED_RANGE_NOT_SATISFIABLE = 416;
  public static final int SC_EXPECTATION_FAILED = 417;

  // public static final int SC_UNPROCESSABLE_ENTITY = 418;

  public static final int SC_INSUFFICIENT_SPACE_ON_RESOURCE = 419;
  public static final int SC_METHOD_FAILURE = 420;
  public static final int SC_UNPROCESSABLE_ENTITY = 422;
  public static final int SC_LOCKED = 423;
  public static final int SC_FAILED_DEPENDENCY = 424;

  // --- 5xx Server Error ---
  public static final int SC_INTERNAL_SERVER_ERROR = 500;
  public static final int SC_NOT_IMPLEMENTED = 501;
  public static final int SC_BAD_GATEWAY = 502;
  public static final int SC_SERVICE_UNAVAILABLE = 503;
  public static final int SC_GATEWAY_TIMEOUT = 504;
  public static final int SC_HTTP_VERSION_NOT_SUPPORTED = 505;

  public static final int SC_INSUFFICIENT_STORAGE = 507;



  // initialize status-reasons array
  static {
    // HTTP 1.0 Server status codes -- see RFC 1945
    addStatus(SC_OK, "OK");
    addStatus(SC_CREATED, "Created");
    addStatus(SC_ACCEPTED, "Accepted");
    addStatus(SC_NO_CONTENT, "No Content");
    addStatus(SC_MOVED_PERMANENTLY, "Moved Permanently");
    addStatus(SC_MOVED_TEMPORARILY, "Moved Temporarily");
    addStatus(SC_NOT_MODIFIED, "Not Modified");
    addStatus(SC_BAD_REQUEST, "Bad Request");
    addStatus(SC_UNAUTHORIZED, "Unauthorized");
    addStatus(SC_FORBIDDEN, "Forbidden");
    addStatus(SC_NOT_FOUND, "Not Found");
    addStatus(SC_INTERNAL_SERVER_ERROR, "Internal Server Error");
    addStatus(SC_NOT_IMPLEMENTED, "Not Implemented");
    addStatus(SC_BAD_GATEWAY, "Bad Gateway");
    addStatus(SC_SERVICE_UNAVAILABLE, "Service Unavailable");

    // HTTP 1.1 Server status codes -- see RFC 2048
    addStatus(SC_CONTINUE, "Continue");
    addStatus(SC_TEMPORARY_REDIRECT, "Temporary Redirect");
    addStatus(SC_METHOD_NOT_ALLOWED, "Method Not Allowed");
    addStatus(SC_CONFLICT, "Conflict");
    addStatus(SC_PRECONDITION_FAILED, "Precondition Failed");
    addStatus(SC_REQUEST_TOO_LONG, "Request Too Long");
    addStatus(SC_REQUEST_URI_TOO_LONG, "Request-URI Too Long");
    addStatus(SC_UNSUPPORTED_MEDIA_TYPE, "Unsupported Media Type");
    addStatus(SC_MULTIPLE_CHOICES, "Multiple Choices");
    addStatus(SC_SEE_OTHER, "See Other");
    addStatus(SC_USE_PROXY, "Use Proxy");
    addStatus(SC_PAYMENT_REQUIRED, "Payment Required");
    addStatus(SC_NOT_ACCEPTABLE, "Not Acceptable");
    addStatus(SC_PROXY_AUTHENTICATION_REQUIRED,
                                "Proxy Authentication Required");
    addStatus(SC_REQUEST_TIMEOUT, "Request Timeout");

    addStatus(SC_SWITCHING_PROTOCOLS, "Switching Protocols");
    addStatus(SC_NON_AUTHORITATIVE_INFORMATION,
                             "Non Authoritative Information");
    addStatus(SC_RESET_CONTENT, "Reset Content");
    addStatus(SC_PARTIAL_CONTENT, "Partial Content");
    addStatus(SC_GATEWAY_TIMEOUT, "Gateway Timeout");
    addStatus(SC_HTTP_VERSION_NOT_SUPPORTED,
                             "Http Version Not Supported");
    addStatus(SC_GONE, "Gone");
    addStatus(SC_LENGTH_REQUIRED, "Length Required");
    addStatus(SC_REQUESTED_RANGE_NOT_SATISFIABLE,
              "Requested Range Not Satisfiable");
    addStatus(SC_EXPECTATION_FAILED, "Expectation Failed");

    // WebDAV Server-specific status codes
    addStatus(SC_PROCESSING, "Processing");
    addStatus(SC_MULTI_STATUS, "Multi-Status");
    addStatus(SC_UNPROCESSABLE_ENTITY, "Unprocessable Entity");
    addStatus(SC_INSUFFICIENT_SPACE_ON_RESOURCE,
                           "Insufficient Space On Resource");
    addStatus(SC_METHOD_FAILURE, "Method Failure");
    addStatus(SC_LOCKED, "Locked");
    addStatus(SC_INSUFFICIENT_STORAGE, "Insufficient Storage");
    addStatus(SC_FAILED_DEPENDENCY, "Failed Dependency");
  }

} // end of HttpStatus class
